package combatmanager;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.Objects;
import java.util.prefs.Preferences;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class CombatManager extends JFrame {
	
	public static final BufferedImage PLUS_IMAGE = loadImage("Plus.png", 20, 20);
    public static final BufferedImage DOWN_ARROW_IMAGE = loadImage("Down Arrow Filled.png", 20, 20);
    public static final BufferedImage UP_ARROW_IMAGE = loadImage("Up Arrow Filled.png", 20, 20);
    public static final BufferedImage THREE_BARS_IMAGE = loadImage("Three Bars.png", 20, 20);
    public static final BufferedImage MAGNIFYING_GLASS_IMAGE = loadImage("Magnifying Glass.png", 20, 20);
    public static final BufferedImage X_IMAGE = loadImage("X.png", 20, 20);
    public static final Pattern EMPTY_STRING_PATTERN = Pattern.compile("[ ]*");

    private static final Insets FIVE_ON_SIDES = new Insets(5, 5, 5, 5);
    private static final Insets NO_INSETS = new Insets(0, 0, 0, 0);

    public static final DocumentFilter digitFilter = new DocumentFilter() {

        @Override
        public void insertString(FilterBypass fb, int offset, String string, AttributeSet attr) throws BadLocationException {
            final StringBuilder sb = new StringBuilder();
            for (char c : string.toCharArray()) {
                if (Character.isDigit(c)) {
                    sb.append(c);
                }
            }
            fb.insertString(offset, sb.toString(), attr);
            checkSize(fb);
        }

        @Override
        public void replace(FilterBypass fb, int offset, int length, String text, AttributeSet attrs) throws BadLocationException {
            final StringBuilder sb = new StringBuilder();
            for (char c : text.toCharArray()) {
                if (Character.isDigit(c)) {
                    sb.append(c);
                }
            }
            fb.replace(offset, length, sb.toString(), attrs);
            checkSize(fb);
        }

        private void checkSize(FilterBypass fb) throws BadLocationException {
            Document d = fb.getDocument();
            if (d.getLength() != 0) {
                String text = d.getText(0, d.getLength());
                try {
                    Integer.parseInt(text);
                } catch (NumberFormatException e) {
                    d.remove(d.getLength() - 1, 1);
                    checkSize(fb);
                }
            }
        }

    };

    private final ComponentAdapter resizeMoveListener = new ComponentAdapter() {
        @Override
        public void componentResized(ComponentEvent e) {
            putObject("mainFrameSize", getSize());
        }

        @Override
        public void componentMoved(ComponentEvent e) {
            putObject("mainFrameLocation", getLocation());
        }
    };

    private final WindowAdapter closeListener = new WindowAdapter() {
        @Override
        public void windowClosing(WindowEvent e) {
            if (enemies.getComponentCount() == 0 || JOptionPane.showConfirmDialog(CombatManager.this, "Are you sure you would like to exit?", "Confirm Exit", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION) {
                System.exit(0);
            }
        }
    };

    private final DocumentListener searchBarListener = new DocumentListener() {
        @Override
        public void insertUpdate(DocumentEvent e) {
            filterResults();
        }

        @Override
        public void removeUpdate(DocumentEvent e) {
            filterResults();
        }

        @Override
        public void changedUpdate(DocumentEvent e) {
            filterResults();
        }
    };

    private final JPanel panel;
    private final JPanel addPanel;
    private final JTextField name;
    private final JTextField health;
    private final JButton showInfoFrame;
    private final JButton add;
    private final JButton filterOpenButton;
    private final JPanel filterPanel;
    private final JTextField searchBar;
    private final JButton filterMenuButton;
    private final JPopupMenu filterMenu;
    private final JCheckBoxMenuItem enemiesCheck;
    private final JCheckBoxMenuItem playerCheck;
    private final JCheckBoxMenuItem deadCheck;
    private final JButton removeVisible;
    private final JButton showVisible;
    private final JButton hideVisible;
    private final JButton frameSettings;
    private final JPopupMenu settingsMenu;
    private final JCheckBoxMenuItem onTop;
    private final JCheckBoxMenuItem resizeable;
    private final JCheckBoxMenuItem visible;
    private final JCheckBoxMenuItem player;
    private final JMenuItem frameSize;
    private final JMenuItem nameSize;
    private int defaultNameSize = 16;
    private final JMenuItem infoSize;
    private int defaultInfoSize = 12;
    private Dimension defaultFrameSize = new Dimension(75, 75);
    private final JPanel settings;
    private final JPanel enemies;
    private String publicInfo;
    private String privateInfo;

    public CombatManager() {
        super("Combat Manager");
        addPanel = new JPanel(new GridBagLayout());
        name = new JTextField();
        health = new JTextField("0");
        ((AbstractDocument) health.getDocument()).setDocumentFilter(digitFilter);
        showInfoFrame = new JButton("Edit Enemy Info");
        showInfoFrame.addActionListener(e -> {
            InfoInputFrame f = new InfoInputFrame(this, publicInfo, privateInfo);
            if (f.getResponse() == InfoInputFrame.OK_RESPONSE) {
                publicInfo = f.getPublic();
                privateInfo = f.getPrivate();
            }
        });
        add = new JButton(new ImageIcon(PLUS_IMAGE));
        add.setPreferredSize(new Dimension(20, 20));
        filterOpenButton = new JButton(new ImageIcon(DOWN_ARROW_IMAGE));
        filterOpenButton.setPreferredSize(new Dimension(20, 20));
        addPanel.add(addLabel("Name:", name), new GridBagConstraints(0, 0, 2, 1, 0.7, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, FIVE_ON_SIDES, 0, 0));
        addPanel.add(addLabel("Health:", health), new GridBagConstraints(2, 0, 1, 1, 0.4, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, FIVE_ON_SIDES, 0, 0));
        addPanel.add(showInfoFrame, new GridBagConstraints(3, 0, 3, 1, 0.9, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, FIVE_ON_SIDES, 0, 0));
        addPanel.add(add, new GridBagConstraints(6, 0, 1, 1, 0.3, 0.0, GridBagConstraints.LINE_START, GridBagConstraints.NONE, NO_INSETS, 0, 0));
        addPanel.add(filterOpenButton, new GridBagConstraints(7, 0, 1, 1, 0.3, 0.0, GridBagConstraints.LINE_END, GridBagConstraints.NONE, FIVE_ON_SIDES, 0, 0));
        searchBar = new JTextField();
        searchBar.getDocument().addDocumentListener(searchBarListener);
        filterMenuButton = new JButton(new ImageIcon(MAGNIFYING_GLASS_IMAGE));
        filterMenuButton.setPreferredSize(new Dimension(20, 20));
        filterMenu = new JPopupMenu();
        filterMenuButton.addActionListener(e -> filterMenu.show(filterMenuButton, 0, 0));
        enemiesCheck = new JCheckBoxMenuItem("Enemies", true);
        enemiesCheck.addActionListener(e -> filterResults());
        playerCheck = new JCheckBoxMenuItem("Players", true);
        playerCheck.addActionListener(e -> filterResults());
        deadCheck = new JCheckBoxMenuItem("Dead", true);
        deadCheck.addActionListener(e -> filterResults());
        filterMenu.add(enemiesCheck);
        filterMenu.add(playerCheck);
        filterMenu.add(deadCheck);
        removeVisible = new JButton("Delete Visible");
        showVisible = new JButton("Show Visible");
        hideVisible = new JButton("Hide Visible");
        frameSettings = new JButton(new ImageIcon(THREE_BARS_IMAGE));
        frameSettings.setPreferredSize(new Dimension(20, 20));
        settingsMenu = new JPopupMenu();
        frameSettings.addActionListener(e -> settingsMenu.show(frameSettings, 0, 0));
        onTop = new JCheckBoxMenuItem("Always on Top");
        resizeable = new JCheckBoxMenuItem("Resizeable");
        visible = new JCheckBoxMenuItem("Visible");
        player = new JCheckBoxMenuItem("Player");
        frameSize = new JMenuItem("Frame Size: " + defaultFrameSize.width + "x" + defaultFrameSize.height);
        frameSize.addActionListener(e -> {
            Size2DDialog d = new Size2DDialog(this, "Enter default frame size:", defaultFrameSize.width, defaultFrameSize.height);
            if (d.getResult() == Size2DDialog.OK_RESULT) {
                defaultFrameSize = d.getInputSize();
                frameSize.setText("Frame Size: " + defaultFrameSize.width + "x" + defaultFrameSize.height);
            }
        });
        nameSize = new JMenuItem("Name Size: " + defaultNameSize);
        nameSize.addActionListener(e -> {
            Size1DDialog d = new Size1DDialog(this, "Enter default name size:", defaultNameSize);
            if (d.getResult() == Size1DDialog.OK_RESULT) {
                defaultNameSize = d.getInputSize();
                nameSize.setText("Name Size: " + defaultNameSize);
            }
        });
        infoSize = new JMenuItem("Info Size: " + defaultInfoSize);
        infoSize.addActionListener(e -> {
            Size1DDialog d = new Size1DDialog(this, "Enter default info size:", defaultInfoSize);
            if (d.getResult() == Size1DDialog.OK_RESULT) {
                defaultInfoSize = d.getInputSize();
                infoSize.setText("Info Size: " + defaultInfoSize);
            }
        });
        settingsMenu.add(onTop);
        settingsMenu.add(resizeable);
        settingsMenu.add(visible);
        settingsMenu.addSeparator();
        settingsMenu.add(player);
        settingsMenu.addSeparator();
        settingsMenu.add(frameSize);
        settingsMenu.add(nameSize);
        settingsMenu.add(infoSize);
        filterPanel = new JPanel(new GridBagLayout());
        filterPanel.add(addLabel("Search:", searchBar), new GridBagConstraints(0, 0, 3, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, FIVE_ON_SIDES, 0, 0));
        filterPanel.add(filterMenuButton, new GridBagConstraints(3, 0, 1, 1, 0.3, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, FIVE_ON_SIDES, 0, 0));
        filterPanel.add(removeVisible, new GridBagConstraints(4, 0, 1, 1, 0.3, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, FIVE_ON_SIDES, 0, 0));
        filterPanel.add(showVisible, new GridBagConstraints(5, 0, 1, 1, 0.3, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, FIVE_ON_SIDES, 0, 0));
        filterPanel.add(hideVisible, new GridBagConstraints(6, 0, 1, 1, 0.3, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, FIVE_ON_SIDES, 0, 0));
        filterPanel.add(frameSettings, new GridBagConstraints(7, 0, 1, 1, 0.3, 0.0, GridBagConstraints.LINE_END, GridBagConstraints.NONE, FIVE_ON_SIDES, 0, 0));
        filterPanel.setVisible(false);
        filterOpenButton.addActionListener(e -> {
            if (filterPanel.isVisible()) {
                filterOpenButton.setIcon(new ImageIcon(DOWN_ARROW_IMAGE));
                filterPanel.setVisible(false);
            } else {
                filterOpenButton.setIcon(new ImageIcon(UP_ARROW_IMAGE));
                filterPanel.setVisible(true);
            }
        });
        settings = new JPanel(new BorderLayout());
        settings.add(addPanel, BorderLayout.PAGE_START);
        settings.add(filterPanel, BorderLayout.PAGE_END);
        enemies = new JPanel();
        removeVisible.addActionListener(e -> {
            if (JOptionPane.showConfirmDialog(this, "Are your sure you would like do delete all visible enemies?", "Delete Enemies", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION) {
                for (Component c : enemies.getComponents()) {
                    if (c instanceof EnemyEntry) {
                        EnemyEntry en = (EnemyEntry) c;
                        if (en.isVisible()) {
                            en.dispose();
                            enemies.remove(en);
                        }
                    }
                }
                enemies.revalidate();
                enemies.repaint();
            }
        });
        showVisible.addActionListener(e -> {
            for (Component c : enemies.getComponents()) {
                if (c instanceof EnemyEntry) {
                    EnemyEntry en = (EnemyEntry) c;
                    if (en.isVisible() && !en.isFrameVisible()) {
                        en.setFrameVisible(true);
                    }
                }
            }
        });
        hideVisible.addActionListener(e -> {
            for (Component c : enemies.getComponents()) {
                if (c instanceof EnemyEntry) {
                    EnemyEntry en = (EnemyEntry) c;
                    if (en.isVisible() && en.isFrameVisible()) {
                        en.setFrameVisible(false);
                    }
                }
            }
        });
        add.addActionListener(e -> {
            int maxHealth = 0;
            if (health.getText().length() != 0) {
                maxHealth = Integer.parseInt(health.getText());
            }
            String name = "Unnamed";
            if (!EMPTY_STRING_PATTERN.matcher(this.name.getText()).matches()) {
                name = this.name.getText();
            }
            enemies.add(new EnemyEntry(name, maxHealth, publicInfo, privateInfo, onTop.isSelected(), resizeable.isSelected(), visible.isSelected(), player.isSelected(), defaultFrameSize, defaultNameSize, defaultInfoSize));
            enemies.revalidate();
        });
        BoxLayout enemyLayout = new BoxLayout(enemies, BoxLayout.Y_AXIS);
        enemies.setLayout(enemyLayout);
        JScrollPane enemiesScroll = new JScrollPane(enemies);
        panel = new JPanel(new BorderLayout());
        panel.add(settings, BorderLayout.PAGE_START);
        panel.add(enemiesScroll, BorderLayout.CENTER);
        setContentPane(panel);
        setSize(getObject("mainFrameSize", new Dimension(500, 500)));
        setLocation(getObject("mainFrameLocation", new Point(0, 0)));
        addComponentListener(resizeMoveListener);
        setMinimumSize(new Dimension(750, 300));
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(closeListener);
        setVisible(true);
    }

    private void filterResults() {
        String search = EMPTY_STRING_PATTERN.matcher(searchBar.getText()).matches() ? null : searchBar.getText().replace(" ", "").toLowerCase();
        Pattern p = search == null ? null : Pattern.compile(Pattern.quote(search));
        for (Component c : enemies.getComponents()) {
            if (c instanceof EnemyEntry) {
                EnemyEntry e = (EnemyEntry) c;
                Matcher m = null;
                if (p != null) {
                    String data = e.getName().trim().replace(" ", "").toLowerCase() + " " + e.getPublicInfo().trim().replace(" ", "").toLowerCase() + " " + e.getHiddenInfo().trim().replace(" ", "").toLowerCase();
                    m = p.matcher(data);
                }
                e.setVisible((m == null || m.find()) && ((e.isPlayer() && playerCheck.isSelected()) || (!e.isPlayer() && enemiesCheck.isSelected())) && (deadCheck.isSelected() || e.getHealth() > 0));
            }
        }
    }

    public static final Preferences PREFERENCES = Preferences.userRoot().node("dndCombatManager");

    public static void main(String[] args) throws ClassNotFoundException, UnsupportedLookAndFeelException, InstantiationException, IllegalAccessException {
        UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
        new CombatManager();
    }

    @SuppressWarnings("unchecked")
	public static <T> T getObject(String key, T def) {
        byte[] dummy = new byte[0];
        byte[] data = PREFERENCES.getByteArray(key, dummy);
        if (data == dummy) {
            return def;
        }
        try {
            ObjectInputStream in = new ObjectInputStream(new ByteArrayInputStream(data));
            return (T) in.readObject();
        } catch (IOException | ClassCastException | ClassNotFoundException e) {
            return def;
        }
    }

    public static void putObject(String key, Object obj) {
        try {
            ByteArrayOutputStream bytes = new ByteArrayOutputStream();
            ObjectOutputStream out = new ObjectOutputStream(bytes);
            out.writeObject(Objects.requireNonNull(obj));
            PREFERENCES.putByteArray(key, bytes.toByteArray());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static BufferedImage loadImage(String name, int width, int height) {
        try {
            BufferedImage img = ImageIO.read(ClassLoader.getSystemResource(name));
            width = width < 1 ? img.getWidth() : width;
            height = height < 1 ? img.getHeight() : height;
            BufferedImage newImg = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
            Graphics2D g = newImg.createGraphics();
            g.drawImage(img.getScaledInstance(width, height, BufferedImage.SCALE_SMOOTH), 0, 0, null);
            g.dispose();
            return newImg;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static JComponent addLabel(String text, JComponent component) {
        return addLabel(new JLabel(text), component, 3);
    }

    public static JComponent addLabel(JLabel text, JComponent component, int pad) {
        JPanel panel = new JPanel(new GridBagLayout());
        panel.setOpaque(false);
        panel.add(text, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, pad), 0, 0));
        panel.add(component, new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, NO_INSETS, 0, 0));
        return panel;
    }

}
