package combatmanager;

import javax.swing.*;
import javax.swing.border.MatteBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.AbstractDocument;
import java.awt.*;
import java.awt.event.*;

public class EnemyEntry extends JPanel {

    private static final Insets FIVE_ON_SIDES = new Insets(5, 5, 5, 5);

    private final DocumentListener healthUpdateListener = new DocumentListener() {
        @Override
        public void insertUpdate(DocumentEvent e) {
            update();
        }

        @Override
        public void removeUpdate(DocumentEvent e) {
            update();
        }

        @Override
        public void changedUpdate(DocumentEvent e) {
            update();
        }

        private void update() {
            if (getHealth() <= getMaxHealth() / 2) {
                setBackground(Color.red);
                frame.setBackground(Color.red);
            } else {
                Color c = UIManager.getColor("Panel.background");
                setBackground(c);
                frame.setBackground(c);
            }
        }
    };

    private final DocumentListener nameListener = new DocumentListener() {
        @Override
        public void insertUpdate(DocumentEvent e) {
            update();
        }

        @Override
        public void removeUpdate(DocumentEvent e) {
            update();
        }

        @Override
        public void changedUpdate(DocumentEvent e) {
            update();
        }

        private void update() {
            frame.setTitle(name.getText());
        }
    };

    private final ComponentAdapter frameSizeListener = new ComponentAdapter() {
        @Override
        public void componentResized(ComponentEvent e) {
            frameSize.setText("Frame Size: " + e.getComponent().getWidth() + "x" + e.getComponent().getHeight());
        }

        @Override
        public void componentHidden(ComponentEvent e) {
            visible.setSelected(false);
        }
    };

    private MouseAdapter moveListener = new MouseAdapter() {
        private int x;
        private int y;

        @Override
        public void mouseClicked(MouseEvent e) {
            if (e.isShiftDown()) {
                resizeable.setSelected(!resizeable.isSelected());
                EnemyFrame f = new EnemyFrame(name.getText(), publicInfo, onTop.isSelected(), resizeable.isSelected(), visible.isSelected(), player.isSelected(), frame.getNameSize(), frame.getInfoSize());
                f.setSize(frame.getSize());
                f.setLocation(frame.getLocation());
                f.addMouseListener(moveListener);
                f.addMouseMotionListener(moveListener);
                f.getInfo().addMouseListener(moveListener);
                f.addComponentListener(frameSizeListener);
                frame.dispose();
                frame = f;
            }
        }

        @Override
        public void mousePressed(MouseEvent e) {
            if (!e.isShiftDown() && e.getSource() == frame) {
                x = e.getX();
                y = e.getY();
            }
        }

        @Override
        public void mouseDragged(MouseEvent e) {
            if (!e.isShiftDown() && e.getSource() == frame) {
                frame.setLocation(e.getXOnScreen() - x, e.getYOnScreen() - y);
            }
        }
    };

    private final JTextField name;
    private final JTextField health;
    private final JTextField maxHealth;
    private final JButton showInfoFrame;
    private final JButton copy;
    private final JButton remove;
    private EnemyFrame frame;
    private final JPopupMenu frameMenu;
    private final JCheckBoxMenuItem onTop;
    private final JCheckBoxMenuItem resizeable;
    private final JCheckBoxMenuItem visible;
    private final JCheckBoxMenuItem player;
    private final JMenuItem frameSize;
    private final JMenuItem nameSize;
    private final JMenuItem infoSize;
    private final JButton frameMenuButton;
    private String publicInfo;
    private String privateInfo;

    public EnemyEntry(String name, int maxHealth, String publicInfo, String privateInfo, boolean onTop, boolean resizeable, boolean visible, boolean player, Dimension size, int nameSize, int infoSize) {
        super(new GridBagLayout());
        setBorder(new MatteBorder(1, 1, 1, 1, Color.lightGray));
        this.name = new JTextField(notNull(name), 7);
        this.name.getDocument().addDocumentListener(nameListener);
        this.health = new JTextField(String.valueOf(maxHealth), 3);
        this.health.getDocument().addDocumentListener(healthUpdateListener);
        ((AbstractDocument) this.health.getDocument()).setDocumentFilter(CombatManager.digitFilter);
        this.maxHealth = new JTextField(String.valueOf(maxHealth), 3);
        this.maxHealth.getDocument().addDocumentListener(healthUpdateListener);
        ((AbstractDocument) this.maxHealth.getDocument()).setDocumentFilter(CombatManager.digitFilter);
        this.publicInfo = publicInfo == null ? "" : publicInfo;
        this.privateInfo = privateInfo == null ? "" : privateInfo;
        showInfoFrame = new JButton("Edit Enemy Info");
        showInfoFrame.addActionListener(e -> {
            InfoInputFrame f = new InfoInputFrame(SwingUtilities.getWindowAncestor(this), this.publicInfo, this.privateInfo);
            if (f.getResponse() == InfoInputFrame.OK_RESPONSE) {
                this.publicInfo = f.getPublic();
                this.privateInfo = f.getPrivate();
                frame.setText(this.publicInfo);
            }
        });
        copy = new JButton(new ImageIcon(CombatManager.PLUS_IMAGE));
        copy.setPreferredSize(new Dimension(20, 20));
        remove = new JButton(new ImageIcon(CombatManager.X_IMAGE));
        remove.setPreferredSize(new Dimension(20, 20));
        frame = new EnemyFrame(name, publicInfo, onTop, resizeable, visible, player, nameSize, infoSize);
        frame.setSize(size);
        frame.addComponentListener(frameSizeListener);
        frame.addMouseListener(moveListener);
        frame.addMouseMotionListener(moveListener);
        frame.getInfo().addMouseListener(moveListener);
        remove.addActionListener(e -> {
            if (JOptionPane.showConfirmDialog(SwingUtilities.getWindowAncestor(this), "Are you sure you want to remove this enemy?", "Confirm Remove", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION) {
                frame.dispose();
                Container parent = getParent();
                parent.remove(this);
                parent.revalidate();
                parent.repaint();
            }
        });
        frameMenu = new JPopupMenu();
        this.onTop = new JCheckBoxMenuItem("Always on Top", onTop);
        this.onTop.addActionListener(e -> frame.setAlwaysOnTop(this.onTop.isSelected()));
        this.resizeable = new JCheckBoxMenuItem("Resizeable", resizeable);
        this.visible = new JCheckBoxMenuItem("Visible", visible);
        this.visible.addActionListener(e -> frame.setVisible(this.visible.isSelected()));
        this.player = new JCheckBoxMenuItem("Player", player);
        this.player.addActionListener(e -> frame.setPlayer(this.player.isSelected()));
        this.frameSize = new JMenuItem("Frame Size: " + frame.getWidth() + "x" + frame.getHeight());
        this.frameSize.addActionListener(e -> {
            Size2DDialog d = new Size2DDialog(SwingUtilities.getWindowAncestor(this), "Enter new frame size:", frame.getWidth(), frame.getHeight());
            if (d.getResult() == Size2DDialog.OK_RESULT) {
                frame.setSize(d.getInputSize());
            }
        });
        this.nameSize = new JMenuItem("Name Size: " + nameSize);
        this.nameSize.addActionListener(e -> {
            Size1DDialog d = new Size1DDialog(SwingUtilities.getWindowAncestor(this), "Enter name size:", frame.getNameSize());
            if (d.getResult() == Size1DDialog.OK_RESULT) {
                int s = d.getInputSize();
                frame.setNameSize(s);
                this.nameSize.setText("Name Size: " + s);
            }
        });
        this.infoSize = new JMenuItem("Info Size: " + infoSize);
        this.infoSize.addActionListener(e -> {
            Size1DDialog d = new Size1DDialog(SwingUtilities.getWindowAncestor(this), "Enter info size:", frame.getInfoSize());
            if (d.getResult() == Size1DDialog.OK_RESULT) {
                int s = d.getInputSize();
                frame.setInfoSize(s);
                this.infoSize.setText("Info Size: " + s);
            }
        });
        this.resizeable.addActionListener(e -> {
            EnemyFrame f = new EnemyFrame(this.name.getText(), this.publicInfo, this.onTop.isSelected(), this.resizeable.isSelected(), this.visible.isSelected(), this.player.isSelected(), frame.getNameSize(), frame.getInfoSize());
            f.setSize(frame.getSize());
            f.setLocation(frame.getLocation());
            f.addMouseListener(moveListener);
            f.addMouseMotionListener(moveListener);
            f.getInfo().addMouseListener(moveListener);
            f.addComponentListener(frameSizeListener);
            frame.dispose();
            frame = f;
        });
        frameMenu.add(this.onTop);
        frameMenu.add(this.resizeable);
        frameMenu.add(this.visible);
        frameMenu.addSeparator();
        frameMenu.add(this.player);
        frameMenu.addSeparator();
        frameMenu.add(frameSize);
        frameMenu.add(this.nameSize);
        frameMenu.add(this.infoSize);
        frameMenuButton = new JButton(new ImageIcon(CombatManager.THREE_BARS_IMAGE));
        frameMenuButton.setPreferredSize(new Dimension(20, 20));
        frameMenuButton.addActionListener(e -> frameMenu.show(frameMenuButton, 0, 0));
        copy.addActionListener(e -> {
            String newName = this.name.getText();
            if ((e.getModifiers() & InputEvent.SHIFT_DOWN_MASK) != 0) {
                newName = (String) JOptionPane.showInputDialog(SwingUtilities.getWindowAncestor(this), "Enter new name:", "Copy Enemy", JOptionPane.QUESTION_MESSAGE, null, null, this.name.getText());
            }
            if (newName != null) {
                EnemyEntry o = new EnemyEntry(newName, getMaxHealth(), this.publicInfo, this.privateInfo, this.onTop.isSelected(), this.resizeable.isSelected(), this.visible.isSelected(), this.player.isSelected(), frame.getSize(), frame.getNameSize(), frame.getInfoSize());
                o.frame.setSize(frame.getSize());
                getParent().add(o);
                getParent().revalidate();
            }
        });
        add(CombatManager.addLabel("Name:", this.name), new GridBagConstraints(0, 0, 2, 1, 0.6, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, FIVE_ON_SIDES, 0, 0));
        add(CombatManager.addLabel("Health:", health), new GridBagConstraints(2, 0, 2, 1, 0.6, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 0), 0, 0));
        add(new JLabel("/"), new GridBagConstraints(5, 0, 1, 1, 0.3, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 0, 0));
        add(this.maxHealth, new GridBagConstraints(6, 0, 2, 1, 0.6, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 5, 5), 0, 0));
        add(showInfoFrame, new GridBagConstraints(8, 0, 1, 1, 0.3, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, FIVE_ON_SIDES, 0, 0));
        add(copy, new GridBagConstraints(9, 0, 1, 1, 0.3, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, FIVE_ON_SIDES, 0, 0));
        add(remove, new GridBagConstraints(10, 0, 1, 1, 0.3, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, FIVE_ON_SIDES, 0, 0));
        add(frameMenuButton, new GridBagConstraints(11, 0, 1, 1, 0.3, 0.3, GridBagConstraints.CENTER, GridBagConstraints.NONE, FIVE_ON_SIDES, 0, 0));
    }

    public boolean isFrameVisible() {
        return frame.isVisible();
    }

    public void setFrameVisible(boolean visible) {
        frame.setVisible(visible);
    }

    public void dispose() {
        frame.dispose();
    }

    public int getMaxHealth() {
        try {
            return maxHealth.getText().length() == 0 ? 0 : Integer.parseInt(maxHealth.getText());
        } catch (NumberFormatException e) {
            return Integer.MAX_VALUE;
        }
    }

    public int getHealth() {
        try {
            return health.getText().length() == 0 ? 0 : Integer.parseInt(health.getText());
        } catch (NumberFormatException e) {
            return Integer.MAX_VALUE;
        }

    }

    @Override
    public String getName() {
        return name.getText();
    }

    public String getPublicInfo() {
        return publicInfo;
    }

    public String getHiddenInfo() {
        return privateInfo;
    }

    public boolean isPlayer() {
        return player.isSelected();
    }

    @Override
    public Dimension getPreferredSize() {
        return getMaximumSize();
    }

    @Override
    public Dimension getMaximumSize() {
        return new Dimension(getParent().getWidth() - 5, 45);
    }

    private static String notNull(String s) {
        return s == null ? "" : s;
    }

}
