package combatmanager;

import javax.swing.*;
import javax.swing.border.MatteBorder;
import java.awt.*;

public class EnemyFrame extends JFrame {

    private final JPanel panel;
    private final JLabel name;
    private final JTextArea info;

    public EnemyFrame(String name, String info, boolean onTop, boolean resizeable, boolean visible, boolean player, int nameSize, int infoSize) {
        super(name);
        setResizable(false);
        setSize(75, 75);
        setAlwaysOnTop(onTop);
        setResizable(true);
        setUndecorated(!resizeable);
        this.name = new JLabel(name);
        this.name.setFont(new Font(Font.SANS_SERIF, Font.BOLD, nameSize));
        this.name.setBorder(new MatteBorder(0, 0, 1, 0, Color.lightGray));
        if (player) {
            this.name.setOpaque(true);
            this.name.setBackground(Color.cyan);
        }
        this.info = new JTextArea(info);
        this.info.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, infoSize));
        this.info.setDisabledTextColor(this.name.getForeground());
        this.info.setEnabled(false);
        this.info.setLineWrap(true);
        this.info.setBackground(getBackground());
        JScrollPane infoScroll = new JScrollPane(this.info);
        infoScroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        infoScroll.setBorder(null);
        panel = new JPanel(new BorderLayout());
        panel.add(this.name, BorderLayout.PAGE_START);
        panel.add(infoScroll, BorderLayout.CENTER);
        setContentPane(panel);
        setDefaultCloseOperation(HIDE_ON_CLOSE);
        setVisible(visible);
    }

    public void setNameSize(int size) {
        name.setFont(new Font(Font.SANS_SERIF, Font.BOLD, size));
    }

    public void setInfoSize(int size) {
        info.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, size));
    }

    public int getNameSize() {
        return name.getFont().getSize();
    }

    public int getInfoSize() {
        return info.getFont().getSize();
    }

    public JTextArea getInfo() {
        return info;
    }

    @Override
    public void setBackground(Color bgColor) {
        super.setBackground(bgColor);
        if (panel != null) panel.setBackground(bgColor);
        if (info != null) info.setBackground(bgColor);
    }

    public void setText(String text) {
        info.setText(text);
    }

    @Override
    public void setTitle(String title) {
        super.setTitle(title);
        if (name != null) this.name.setText(title);
    }

    public void setPlayer(boolean player) {
        if (player) {
            this.name.setOpaque(true);
            this.name.setBackground(Color.cyan);
        } else {
            this.name.setOpaque(false);
            this.name.setBackground(null);
        }
    }

}
