package combatmanager;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

public class InfoInputFrame extends JDialog {

    private static final Insets FIVE_ON_SIDES = new Insets(5, 5, 5, 5);

    public static final int OK_RESPONSE = 0;
    public static final int CANCEL_RESPONSE = 1;

    private final JPanel panel;
    private final JLabel publicMessage;
    private final JTextArea publicPane;
    private final JLabel privateMessage;
    private final JTextArea privatePane;
    private final JButton ok;
    private final JButton cancel;
    private int response;

    public InfoInputFrame(Window parent, String pm, String hm) {
        super(parent, "Edit Enemy Info");
        setModalityType(ModalityType.APPLICATION_MODAL);
        setAlwaysOnTop(true);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        setMinimumSize(new Dimension(200, 200));
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                response = CANCEL_RESPONSE;
                dispose();
            }
        });
        publicMessage = new JLabel("Public:");
        publicPane = new JTextArea(pm);
        privateMessage = new JLabel("Private:");
        privatePane = new JTextArea(hm);
        ok = new JButton("Ok");
        ok.addActionListener(e -> {
            response = OK_RESPONSE;
            dispose();
        });
        cancel = new JButton("Cancel");
        cancel.addActionListener(e -> {
            response = CANCEL_RESPONSE;
            dispose();
        });
        panel = new JPanel(new GridBagLayout());
        panel.add(publicMessage, new GridBagConstraints(0, 0, GridBagConstraints.REMAINDER, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, FIVE_ON_SIDES, 0, 0));
        panel.add(new JScrollPane(publicPane), new GridBagConstraints(0, 1, GridBagConstraints.REMAINDER, 50, 1.0, 1.0,
                GridBagConstraints.CENTER, GridBagConstraints.BOTH, FIVE_ON_SIDES, 0, 0));
        panel.add(privateMessage, new GridBagConstraints(0, 51, GridBagConstraints.REMAINDER, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, FIVE_ON_SIDES, 0, 0));
        panel.add(new JScrollPane(privatePane), new GridBagConstraints(0, 52, GridBagConstraints.REMAINDER, 50, 1.0,
                1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, FIVE_ON_SIDES, 0, 0));
        panel.add(ok, new GridBagConstraints(0, 102, 1, 1, 0.5, 0.5, GridBagConstraints.CENTER,
                GridBagConstraints.HORIZONTAL, FIVE_ON_SIDES, 0, 0));
        panel.add(cancel, new GridBagConstraints(1, 102, GridBagConstraints.REMAINDER, 1, 0.5, 0.5,
                GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, FIVE_ON_SIDES, 0, 0));
        getRootPane().setDefaultButton(ok);
        setContentPane(panel);
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setSize(screenSize.width / 5, screenSize.height / 4);
        setLocation(parent.getX() + (parent.getWidth() / 2) - (getWidth() / 2),
                parent.getY() + (parent.getHeight() / 2) - (getHeight() / 2));
        setVisible(true);
    }

    public int getResponse() {
        return response;
    }

    public String getPublic() {
        return publicPane.getText();
    }

    public String getPrivate() {
        return privatePane.getText();
    }
}
