package combatmanager;

import javax.swing.*;
import javax.swing.text.AbstractDocument;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

public class Size2DDialog extends JDialog {

    public static final int OK_RESULT = 0;
    public static final int CANCEL_RESULT = 1;

    private static final Insets FIVE_ON_SIDES = new Insets(5, 5, 5, 5);

    private final JPanel panel;
    private final JLabel message;
    private final JTextField width;
    private final JTextField height;
    private final JButton ok;
    private final JButton cancel;
    private int result;

    public Size2DDialog(Window parent, String message, int width, int height) {
        super(parent, "Enter Size");
        setModalityType(ModalityType.APPLICATION_MODAL);
        setAlwaysOnTop(true);
        setResizable(false);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                result = CANCEL_RESULT;
                dispose();
            }
        });
        this.message = new JLabel(message);
        this.width = new JTextField(String.valueOf(width), 4);
        ((AbstractDocument) this.width.getDocument()).setDocumentFilter(CombatManager.digitFilter);
        this.height = new JTextField(String.valueOf(height), 4);
        ((AbstractDocument) this.height.getDocument()).setDocumentFilter(CombatManager.digitFilter);
        ok = new JButton("Ok");
        ok.addActionListener(e -> {
            result = OK_RESULT;
            dispose();
        });
        cancel = new JButton("Cancel");
        cancel.addActionListener(e -> {
            result = CANCEL_RESULT;
            dispose();
        });
        panel = new JPanel(new GridBagLayout());
        panel.add(this.message, new GridBagConstraints(0, 0, GridBagConstraints.REMAINDER, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, FIVE_ON_SIDES, 0, 0));
        panel.add(this.width, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.4, GridBagConstraints.PAGE_START, GridBagConstraints.NONE, new Insets(5, 5, 5, 0), 0, 0));
        panel.add(new JLabel("x"), new GridBagConstraints(1, 1, 1, 1, 0.1, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 0, 0));
        panel.add(this.height, new GridBagConstraints(2, 1, GridBagConstraints.REMAINDER, 1, 0.4, 0.0, GridBagConstraints.PAGE_END, GridBagConstraints.NONE, new Insets(5, 0, 5, 5), 0, 0));
        panel.add(ok, new GridBagConstraints(0, 2, 1, 1, 0.7, 0.0, GridBagConstraints.PAGE_START, GridBagConstraints.NONE, FIVE_ON_SIDES, 0, 0));
        panel.add(cancel, new GridBagConstraints(1, 2, GridBagConstraints.REMAINDER, 1, 0.7, 0.0, GridBagConstraints.PAGE_END, GridBagConstraints.NONE, FIVE_ON_SIDES, 0, 0));
        setContentPane(panel);
        getRootPane().setDefaultButton(ok);
        pack();
        setLocation(parent.getX() + (parent.getWidth() / 2) - (getWidth() / 2), parent.getY() + (parent.getHeight() / 2) - (getHeight() / 2));
        setVisible(true);
    }

    public int getResult() {
        return result;
    }

    public Dimension getInputSize() {
        int width = 1;
        if (this.width.getText().length() != 0) {
            width = Integer.parseInt(this.width.getText());
        }
        int height = 1;
        if (this.height.getText().length() != 0) {
            height = Integer.parseInt(this.height.getText());
        }
        return new Dimension(width == 0 ? 1 : width, height == 0 ? 1 : height);
    }

}
